@@ -44,6 +44,10 @@ from bloom.generators.debian.generate_cmd import prepare_arguments
 
 from bloom.logging import info
 
+from bloom.rosdistro_api import get_index
+
+from bloom.util import get_distro_list_prompt
+
 
 class RosDebianGenerator(DebianGenerator):
     title = 'rosdebian'
@@ -53,7 +57,7 @@ class RosDebianGenerator(DebianGenerator):
     def prepare_arguments(self, parser):
         # Add command line arguments for this generator
         add = parser.add_argument
-        add('rosdistro', help="ROS distro to target (groovy, hydro, etc...)")
+        add('rosdistro', help="ROS distro to target (%s, etc.)" % get_distro_list_prompt())
         return DebianGenerator.prepare_arguments(self, parser)
 
     def handle_arguments(self, args):
@@ -67,7 +71,7 @@ class RosDebianGenerator(DebianGenerator):
         info("Releasing for rosdistro: " + self.rosdistro)
         return ret
 
-    def get_subs(self, package, debian_distro, releaser_history):
+    def get_subs(self, package, debian_distro, releaser_history, deb_inc=0, native=False):
         def fallback_resolver(key, peer_packages, rosdistro=self.rosdistro):
             if key in peer_packages:
                 return [sanitize_package_name(rosify_package_name(key, rosdistro))]
@@ -83,7 +87,46 @@ class RosDebianGenerator(DebianGenerator):
             releaser_history=releaser_history,
             fallback_resolver=fallback_resolver
         )
+        subs['Rosdistro'] = self.rosdistro
         subs['Package'] = rosify_package_name(subs['Package'], self.rosdistro)
+
+        # ROS 2 specific bloom extensions.
+        ros2_distros = [
+            name for name, values in get_index().distributions.items()
+            if values.get('distribution_type') == 'ros2']
+        if self.rosdistro in ros2_distros:
+            # Add ros-workspace package as a dependency to any package other
+            # than ros_workspace and its dependencies.
+            if package.name not in ['ament_cmake_core', 'ament_package', 'ros_workspace']:
+                workspace_pkg_name = rosify_package_name('ros-workspace', self.rosdistro)
+                subs['BuildDepends'].append(workspace_pkg_name)
+                subs['Depends'].append(workspace_pkg_name)
+
+            # Add packages necessary to build vendor typesupport for rosidl_interface_packages to their
+            # build dependencies.
+            if self.rosdistro in ros2_distros and \
+                    self.rosdistro not in ('r2b2', 'r2b3', 'ardent') and \
+                    'rosidl_interface_packages' in [p.name for p in package.member_of_groups]:
+                ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES = [
+                    'rosidl-typesupport-connext-c',
+                    'rosidl-typesupport-connext-cpp',
+                    'rosidl-typesupport-fastrtps-c',
+                    'rosidl-typesupport-fastrtps-cpp',
+                ]
+                # OpenSplice was dropped after Eloquent.
+                # rmw implementations are required as dependencies up to Eloquent.
+                if self.rosdistro in ['bouncy', 'crystal', 'dashing', 'eloquent']:
+                    ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES.extend([
+                        'rmw-connext-cpp',
+                        'rmw-fastrtps-cpp',
+                        'rmw-implementation',
+                        'rmw-opensplice-cpp',
+                        'rosidl-typesupport-opensplice-c',
+                        'rosidl-typesupport-opensplice-cpp',
+                    ])
+
+                subs['BuildDepends'] += [
+                    rosify_package_name(name, self.rosdistro) for name in ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES]
         return subs
 
     def generate_branching_arguments(self, package, branch):
@@ -104,13 +147,16 @@ def rosify_package_name(name, rosdistro):
     return 'ros-{0}-{1}'.format(rosdistro, name)
 
 
-def get_subs(pkg, os_name, os_version, ros_distro):
+def get_subs(pkg, os_name, os_version, ros_distro, deb_inc, native):
     # No fallback_resolver provided because peer packages not considered.
     subs = generate_substitutions_from_package(
         pkg,
         os_name,
         os_version,
-        ros_distro
+        ros_distro,
+        RosDebianGenerator.default_install_prefix + ros_distro,
+        deb_inc=deb_inc,
+        native=native
     )
     subs['Package'] = rosify_package_name(subs['Package'], ros_distro)
     return subs
